/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge;

import by.informatics.jcx.fingy.lang.Interpreter;
import java.awt.Component;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

public abstract class GameEnvironment {

    private Map<String, List<Callback>> callbacks =
    	new HashMap<String, List<Callback>>();

    private AnimationSpeed animationSpeed = AnimationSpeed.NORMAL;

    public abstract Component getVisualComponent();

    public abstract void initInterpreter(Interpreter interpreter);

    public abstract void purgeInterpreter(Interpreter interpreter);

    public abstract ArrayList<String> getFunctionList();

    public abstract ArrayList<String> getConstantList();

    public abstract void resetState();

    public abstract void loadFromXML(Node node);

    public abstract Element convertToXML();

    public void setAnimationSpeed(AnimationSpeed speed) {
        animationSpeed = speed;
    }

    public AnimationSpeed getAnimationSpeed() {
        return animationSpeed;
    }

    public boolean canHandleCallback(String name) {
        return false;
    }

    public void registerCallback(String name, Callback callback) {
        if (!callbacks.containsKey(name)) {
            callbacks.put(name, new ArrayList<Callback>());
        }
        callbacks.get(name).add(callback);
    }

    protected void sendCallback(String name, Object arg) {
        List<Callback> ls = callbacks.get(name);
        for (Callback c : ls) {
            c.invoke(this, name, arg);
        }
    }
}
