/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.robot;

import java.awt.Color;

/**
 *
 * @author crome
 */
public class Robot
{
    /* Angle of robot direction (in degree). */

    private double angle = 0;
    private int x = 0;
    private int y = 0;
    private Color color;
    private boolean isBellyDown = true;

    public Robot(int x, int y, double angle, Color trackColor)
    {
        this.x = x;
        this.y = y;
        this.angle = angle;
        this.color = trackColor;
    }

    /**
     * @param angle
     * Specifies rotation in degree in clockwise direction.
     * Negative value means counterclockwise rotation.
     */
    public void turnClockwise(double angle)
    {
        this.angle += angle;
        this.angle %= 360;
    }

    public void goForward(int distance)
    {
        x += Math.round(distance * Math.cos(Math.toRadians(angle)));
        y += Math.round(distance * Math.sin(Math.toRadians(angle)));
    }

    public void setTrackColor(Color color)
    {
        this.color = color;
    }

    public Color getTrackColor()
    {
        return this.color;
    }

    public double getX()
    {
        return x;
    }

    public void setX(int x)
    {
        this.x = x;
    }

    public double getY()
    {
        return y;
    }

    public void setY(int y)
    {
        this.y = y;
    }

    public double getAngle()
    {
        return angle;
    }

    public void setAngle(double angle)
    {
        this.angle = angle;
    }

    public boolean isBellyDown()
    {
        return isBellyDown;
    }

    public void setBellyToDown(boolean b)
    {
        isBellyDown = b;
    }
}
