/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.robot;

import by.informatics.jcx.fingy.lang.*;


public class RobotFunctionInitializer implements Function
{
    private RobotFunction function = null;
    private RobotGameEnvironment robot;
    public RobotFunctionInitializer(RobotGameEnvironment t, RobotFunction f)
    {
        function = f;
        robot    = t;
    }

    @Override
    public Value invoke(VMContext ctx, VList args) throws VMError
    {
        switch (function)
        {
            case TAKE_FLAG:
                robot.takeFlag();
                return NullValue.NULL;
            case PUT_FLAG:
                robot.putFlag(args.get(0).getInt());
                return NullValue.NULL;
            case GET_FLAG_NUMBER:
                return IntValue.create(robot.getFlagNumber());
            case FORWARD:
                robot.goForward();
                return NullValue.NULL;
            case RIGHT:
                robot.turnClockwise();
                return NullValue.NULL;
            case LEFT:
                robot.turnCounterclockwise();
                return NullValue.NULL;
            case ROBOTX:
                return IntValue.create(robot.getRobotX());
            case ROBOTY:
                return IntValue.create(robot.getRobotY());
            case ROBOTANGLE:
                return IntValue.create(robot.getRobotDirection());
            case CAN_GO:
                return BoolValue.create(!robot.isWallInFront());
        }
        return NullValue.NULL;
    }
}
