/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.robot;

import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JFrame;
import javax.swing.WindowConstants;

import by.informatics.jcx.fingy.ge.AnimationSpeed;
import java.awt.Dimension;

/**
 * @author crome
 *
 */
public class RobotTest
{
    private final int DEFAULTSPEED  = 54;
    private final int SPEEDDELTA    = 0;
    private final int WIDTHDELTA    = 10;
    private final int HEIGHTDELTA   = 10;

    private String pictureFileName = "res/blot.png";
    private int speed = DEFAULTSPEED;
    private JFrame frame;

    private RobotText robotCoordinatesText = new RobotText("0", 430, 15, Color.MAGENTA, null);

    private RobotGameEnvironment r;
    private int fieldWidth  = 15;
    private int fieldHeight = 10;

    private int width  = (fieldWidth + 1) * 55;
    private int height = (fieldHeight + 2) * 55;

    public RobotTest()
    {
        r = new RobotGameEnvironment(width, height, fieldWidth, fieldHeight);

        frame = new JFrame();
        frame.setContentPane(r.getPanel());
        frame.setVisible(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.setResizable(true);
        frame.setPreferredSize(new Dimension(width, height));
        frame.pack();
        RobotText tt = new RobotText("SPACE, Cursors, 1,2,3, 0, Z,X,C,V,B, P, Q,W, A,S, F " +
            " ", 15, 15, Color.MAGENTA, null);
        tt.setCoordinateType(RobotText.SCREENCOORDINATE);
        r.addTextMessage(tt);
        robotCoordinatesText.setCoordinateType(RobotText.SCREENCOORDINATE);
        r.addTextMessage(robotCoordinatesText);

        for (int i = 2; i < fieldWidth; i++)
        {
            for (int j = 2; j < fieldHeight; j++)
            {
                if (Math.random() < 0.35)
                {
                    r.addWall(i, j, (int)Math.round(8 * Math.random()));
                }
            }
        }

        for (int i = 1; i <= fieldWidth; i++)
        {
            for (int j = 1; j <= fieldHeight; j++)
            {
                if (Math.random() < 0.35)
                {
                    r.addTreasure(i, j, false);
                }
            }
        }

        r.updateScene();

        frame.setLocationRelativeTo(null);
        frame.setFocusable(true);
        frame.addKeyListener(new RobotKeyAdapter());
    }

    public static void main(String[] args)
    {
        new RobotTest();
    }

    public class RobotKeyAdapter extends KeyAdapter
    {
        @Override
        public void keyReleased(KeyEvent e)
        {
            onKeyReleased(e.getKeyCode());
        }

        @Override
        public void keyPressed(KeyEvent e)
        {
            onKeyPressed(e.getKeyCode());
        }
    }

    private void onKeyPressed(int k)
    {
        if (k == KeyEvent.VK_0)
        {
//            segmentCount = 0;
            robotCoordinatesText.setText(" [" +
                Integer.toString(r.getRobotX()) +
                "; " + Integer.toString(r.getRobotY()) + "]");
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.resetState();
                }
            }).start();
        }
        if (k == KeyEvent.VK_1)
        {
            r.setFieldHeight(r.getFieldHeight() + 1);
        }
        if (k == KeyEvent.VK_2)
        {
            r.setFieldHeight(r.getFieldHeight() - 1);
        }
        if (k == KeyEvent.VK_3)
        {
            r.setTrackColor(Color.BLUE);
        }
        if (k == KeyEvent.VK_SPACE)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    if (r.getFlagNumber() == 0)
                    {
                        r.putFlag((int) (666 * Math.random()));
                    }
                    else
                    {
                        r.takeFlag();
                    }
                }
            }).start();
        }
        if (k == KeyEvent.VK_UP)
        {
            if (k == KeyEvent.VK_UP)
            {
                if (speed < 0)
                {
                    speed = DEFAULTSPEED;
                }
                speed += SPEEDDELTA;
            }
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.goForward();
                }
            }).start();
        }
        if (k == KeyEvent.VK_LEFT)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.turnCounterclockwise();
                }
            }).start();
        }
        if (k == KeyEvent.VK_RIGHT)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.turnClockwise();
                }
            }).start();
        }
        if (k == KeyEvent.VK_P)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    /* Magic numbers to draw picture under robot center [temp solution]. */
                    r.drawPicture(pictureFileName, r.getRobotX() - 40, r.getRobotY() - 50);
                }
            }).start();
        }
        if (k == KeyEvent.VK_Z)
        {
            r.setAnimationSpeed(AnimationSpeed.DISABLED);
        }
        if (k == KeyEvent.VK_X)
        {
            r.setAnimationSpeed(AnimationSpeed.SLOW);
        }
        if (k == KeyEvent.VK_C)
        {
            r.setAnimationSpeed(AnimationSpeed.NORMAL);
        }
        if (k == KeyEvent.VK_V)
        {
            r.setAnimationSpeed(AnimationSpeed.FAST);
        }
        if (k == KeyEvent.VK_B)
        {
            r.setAnimationSpeed(AnimationSpeed.VERYFAST);
        }
        if (k == KeyEvent.VK_Q)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.setBackgroundColor(Color.BLACK);
                }
            }).start();
        }
        if (k == KeyEvent.VK_W)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.setBackgroundColor(Color.WHITE);
                }
            }).start();
        }
        if (k == KeyEvent.VK_A)
        {
            width  += WIDTHDELTA;
            height += HEIGHTDELTA;
            r.setSizeBy(WIDTHDELTA, HEIGHTDELTA);
            frame.pack();
        }
        if (k == KeyEvent.VK_S)
        {
            width  -= WIDTHDELTA;
            height -= HEIGHTDELTA;
            r.setSizeBy(-WIDTHDELTA, -HEIGHTDELTA);
            frame.pack();
        }
        if (k == KeyEvent.VK_F)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    r.centeringRobot();
                    r.updateScene();
                }
            }).start();
        }

        robotCoordinatesText.setText(" [" + Integer.toString(r.getRobotX()) +
            "; " + Integer.toString(r.getRobotY()) + "] D: " +
            Integer.toString(r.getRobotDirection()) +
            " Treasures: " + r.getFoundTreasureCount() + "/" + r.getTreasureCount());
    }

    private void onKeyReleased(int k)
    {
        speed = DEFAULTSPEED;

        if (k == KeyEvent.VK_ESCAPE)
        {
            System.exit(0);
        }
    }
}
