/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.turtle;

import by.informatics.jcx.fingy.lang.*;

import java.awt.Color;

public class TurtleFunctionInitializer implements Function
{
    private TurtleFunction function = null;
    private TurtleGameEnvironment turtle;
    public TurtleFunctionInitializer(TurtleGameEnvironment t, TurtleFunction f)
    {
        function = f;
        turtle   = t;
    }

    @Override
    public Value invoke(VMContext ctx, VList args) throws VMError
    {
        switch (function)
        {
            case LIFTUPBELLY:
                turtle.liftUpBelly();
                return NullValue.NULL;
            case LIFTDOWNBELLY:
                turtle.liftDownBelly();
                return NullValue.NULL;
            case GOFORWARD:
                turtle.goForward(args.get(0).getDouble());
                return NullValue.NULL;
            case ISBELLYDOWN:
                return BoolValue.create(turtle.isBellyDown());
            case DRAWPICTURE:
                // TODO: Implement method
                return NullValue.NULL;
            case DRAWPLOT:
                turtle.drawPlot(args.get(0).getDouble());
                return NullValue.NULL;
            case DRAWTEXT:
                turtle.drawText(args.get(0).getString());
                return NullValue.NULL;
            case GETTRACKCOLOR:
                return IntValue.create(turtle.getTrackColor().getRGB());
            case SETTRACKCOLOR:
                if (args.size() == 1)
                {
                    turtle.setTrackColor(new Color(args.get(0).getInt()));
                }
                else if (args.size() == 3)
                {
                    turtle.setTrackColor(new Color(args.get(0).getInt(),
                        args.get(1).getInt(), args.get(2).getInt()));
                }
                return NullValue.NULL;
            case TURNCLOCKWISE:
                turtle.turnClockwise(args.get(0).getDouble());
                return NullValue.NULL;
            case TURNCOUNTERCLOCKWISE:
                turtle.turnCounterclockwise(args.get(0).getDouble());
                return NullValue.NULL;
            case TURTLEX:
                return FloatValue.create(turtle.getTurtleX());
            case TURTLEY:
                return FloatValue.create(turtle.getTurtleY());
            case TURTLEANGLE:
                return FloatValue.create(turtle.getTurtleAngle());
        }
        return NullValue.NULL;
    }
}
