/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.turtle;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

public class TurtlePanel extends JPanel
{
    private static final long serialVersionUID = 1L;

    private int xViewOffset = 0;
    private int yViewOffset = 0;

    private final int INCREASEDELTA = 200;
    private final int LOWERTHREASHOLD = 2 * INCREASEDELTA;
    private int maxWidth;
    private int maxHeight;

    protected Graphics2D g;
    protected BufferedImage b;

    public TurtlePanel(int width, int height)
    {
        recreateBackImage(width, height);
        setDefaultViewOffset(width, height);
    }

    @Override
    public void paint(Graphics g)
    {
        int w = getWidth();
        int h = getHeight();

        if (w > maxWidth || w + LOWERTHREASHOLD < maxWidth ||
            h > maxHeight || h + LOWERTHREASHOLD < maxHeight)
        {
            recreateBackImage(w, h);
        }

        g.drawImage(b, 0, 0, w, h, xViewOffset, yViewOffset,
            w + xViewOffset, h + yViewOffset, null);
    }

    public void recreateBackImage(int width, int height)
    {
        maxWidth  = width  + INCREASEDELTA;
        maxHeight = height + INCREASEDELTA;

        b = new BufferedImage(maxWidth, maxHeight, BufferedImage.TYPE_INT_ARGB);
        g = (Graphics2D) b.getGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
            RenderingHints.VALUE_INTERPOLATION_BILINEAR);
    }

    public Graphics2D getGraphics2D()
    {
        return g;
    }

    public int getMaxWidth()
    {
        return maxWidth;
    }

    public int getMaxHeight()
    {
        return maxHeight;
    }

    public int getXView()
    {
        return xViewOffset;
    }

    public int getYView()
    {
        return yViewOffset;
    }

    public void setXView(int x)
    {
        xViewOffset = x;
    }

    public void setYView(int y)
    {
        yViewOffset = y;
    }

    public void setDefaultViewOffset(int w, int h)
    {
        xViewOffset = (maxWidth - w) / 2;
        yViewOffset = (maxHeight - h) / 2;
    }

    public void setDefaultViewOffset()
    {
        setDefaultViewOffset(getWidth(), getHeight());
    }
}
