/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.turtle;

import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JFrame;
import javax.swing.WindowConstants;

import by.informatics.jcx.fingy.ge.AnimationSpeed;

/**
 * @author crome
 *
 */
public class TurtleTest
{
    private final int PLOTRADIUS    = 6;
    private final int ROTATIONANGLE = 15;
    private final int DEFAULTSPEED  = 64;
    private final int SPEEDDELTA    = 1;
    private final int WIDTHDELTA    = 10;
    private final int HEIGHTDELTA   = 10;

    private String pictureFileName = "res/blot.png";
    private int speed = DEFAULTSPEED;
    private JFrame frame;
    private int width  = 500;
    private int height = 500;
    private int segmentCount = 0;

    private TurtleText segmentCountText = new TurtleText("0", 460, 15, Color.MAGENTA, null);

    private TurtleGameEnvironment t = new TurtleGameEnvironment(width, height);

    public TurtleTest()
    {
        frame = new JFrame();
        frame.setContentPane(t.getPanel());
        frame.setVisible(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.setResizable(true);
        frame.pack();
        TurtleText tt = new TurtleText("SPACE, Cursors, 1,2,3, 0, Z,X,C,V,B, P, Q,W, A,S, F " +
            "Segments: ", 15, 15, Color.MAGENTA, null);
        tt.setCoordinateType(TurtleText.SCREENCOORDINATE);
        t.addTextMessage(tt);
        segmentCountText.setCoordinateType(TurtleText.SCREENCOORDINATE);
        t.addTextMessage(segmentCountText);

        frame.setLocationRelativeTo(null);
        frame.setFocusable(true);
        frame.addKeyListener(new TurtleKeyAdapter());
    }

    public static void main(String[] args)
    {
        new TurtleTest();
    }

    public class TurtleKeyAdapter extends KeyAdapter
    {
        @Override
        public void keyReleased(KeyEvent e)
        {
            onKeyReleased(e.getKeyCode());
        }

        @Override
        public void keyPressed(KeyEvent e)
        {
            onKeyPressed(e.getKeyCode());
        }
    }

    private void onKeyPressed(int k)
    {
        if (k == KeyEvent.VK_0)
        {
            segmentCount = 0;
            segmentCountText.setText(Integer.toString(segmentCount));
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.resetState();
                }
            }).start();
        }
        if (k == KeyEvent.VK_1)
        {
            t.setTrackColor(Color.RED);
        }
        if (k == KeyEvent.VK_2)
        {
            t.setTrackColor(Color.GREEN);
        }
        if (k == KeyEvent.VK_3)
        {
            t.setTrackColor(Color.BLUE);
        }
        if (k == KeyEvent.VK_SPACE)
        {
            if (t.isBellyDown())
            {
                t.liftUpBelly();
            }
            else
            {
                t.liftDownBelly();
            }
        }
        if (k == KeyEvent.VK_UP || k == KeyEvent.VK_DOWN)
        {
            if (k == KeyEvent.VK_UP)
            {
                if (speed < 0)
                {
                    speed = DEFAULTSPEED;
                }
                speed += SPEEDDELTA;
            }
            if (k == KeyEvent.VK_DOWN)
            {
                if (speed > 0)
                {
                    speed = -DEFAULTSPEED;
                }
                speed -= SPEEDDELTA;
            }
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.goForward(speed);
                    if (t.isBellyDown())
                    {
                        segmentCount++;
                        segmentCountText.setText(Integer.toString(segmentCount));
                        t.drawPlot(PLOTRADIUS);
                    }
                }
            }).start();
        }
        if (k == KeyEvent.VK_LEFT)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.turnCounterclockwise(ROTATIONANGLE);
                }
            }).start();
        }
        if (k == KeyEvent.VK_RIGHT)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.turnClockwise(ROTATIONANGLE);
                }
            }).start();
        }
        if (k == KeyEvent.VK_P)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    /* Magic numbers to draw picture under turtle center [temp solution]. */
                    t.drawPicture(pictureFileName, t.getTurtleX() - 40, t.getTurtleY() - 50);
                }
            }).start();
        }
        if (k == KeyEvent.VK_Z)
        {
            t.setAnimationSpeed(AnimationSpeed.DISABLED);
        }
        if (k == KeyEvent.VK_X)
        {
            t.setAnimationSpeed(AnimationSpeed.SLOW);
        }
        if (k == KeyEvent.VK_C)
        {
            t.setAnimationSpeed(AnimationSpeed.NORMAL);
        }
        if (k == KeyEvent.VK_V)
        {
            t.setAnimationSpeed(AnimationSpeed.FAST);
        }
        if (k == KeyEvent.VK_B)
        {
            t.setAnimationSpeed(AnimationSpeed.VERYFAST);
        }
        if (k == KeyEvent.VK_Q)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.setBackgroundColor(Color.BLACK);
                }
            }).start();
        }
        if (k == KeyEvent.VK_W)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.setBackgroundColor(Color.WHITE);
                }
            }).start();
        }
        if (k == KeyEvent.VK_A)
        {
            width  += WIDTHDELTA;
            height += HEIGHTDELTA;
            t.setSizeBy(WIDTHDELTA, HEIGHTDELTA);
            frame.pack();
        }
        if (k == KeyEvent.VK_S)
        {
            width  -= WIDTHDELTA;
            height -= HEIGHTDELTA;
            t.setSizeBy(-WIDTHDELTA, -HEIGHTDELTA);
            frame.pack();
        }
        if (k == KeyEvent.VK_F)
        {
            new Thread(new Runnable()
            {
                @Override
                public void run()
                {
                    t.centeringTurtle();
                }
            }).start();
        }
    }

    private void onKeyReleased(int k)
    {
        speed = DEFAULTSPEED;

        if (k == KeyEvent.VK_ESCAPE)
        {
            System.exit(0);
        }
    }
}
