/**
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Konstantin Svirsky aka crome <mailto:c@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ge.turtle;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;

/**
 * @author crome
 *
 */
public class TurtleText extends TurtleDrawableFigure
{
    public static final int GLOBALCOORDINATE = 0;
    public static final int SCREENCOORDINATE = 1;

    private final Font defaultFont = new Font(Font.DIALOG, Font.BOLD, 15);
    private String text;

    private int coordinateType = GLOBALCOORDINATE;
    private Color color;
    private Font font;

    private double x;
    private double y;

    public TurtleText(String text, double x, double y, Color color, Font font)
    {
        this.text  = text;
        this.x     = x;
        this.y     = y;
        this.color = color;
        this.font  = font;
        if (font == null)
        {
            this.font = defaultFont;
        }
    }

    public String getText()
    {
        return text;
    }

    public double getX()
    {
        return x;
    }

    public double getY()
    {
        return y;
    }

    public Color getColor()
    {
        return color;
    }

    public Font getFont()
    {
        return font;
    }

    public void setText(String t)
    {
        text = t;
    }

    public void setCoordinateType(int t)
    {
        coordinateType = t;
    }

    public int getCoordinateType()
    {
        return coordinateType;
    }

    @Override
    public void draw(Graphics2D g, int xOffset, int yOffset)
    {
        g.setFont(font);
        g.drawString(text, (int) x + xOffset, (int) y + yOffset);
    }
}
