/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package by.informatics.jcx.fingy.ide;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.KeyEventPostProcessor;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.awt.event.KeyEvent;
import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;

/**
 * The main class of the application.
 */
public class FingyApp extends SingleFrameApplication {

    /**
     * At startup create and show the main frame of the application.
     */
    @Override
    protected void startup() {
        FingyView view = new FingyView(this);
        // dispatch all unhandled events to main frame
        setupKeyEventsPostProcessor(view.getComponent());
        show(view);
        view.arrangeWindows();
    }

    protected void setupKeyEventsPostProcessor(final Component target) {
        final KeyboardFocusManager km =
                KeyboardFocusManager.getCurrentKeyboardFocusManager();
        km.addKeyEventPostProcessor(new KeyEventPostProcessor() {

            @Override
            public boolean postProcessKeyEvent(KeyEvent e) {

                // disable global shortcuts in modal dialogs
                Window aw = km.getActiveWindow();
                if (aw instanceof Dialog) {
                    Dialog.ModalityType mt = ((Dialog) aw).getModalityType();
                    if (mt != Dialog.ModalityType.MODELESS) {
                        return false;
                    }
                }

                if (!e.isConsumed()) {
                    km.redispatchEvent(target, e);
                    return true;
                }
                return false;
            }
        });
    }

    /**
     * This method is to initialize the specified window by injecting resources.
     * Windows shown in our application come fully initialized from the GUI
     * builder, so this additional configuration is not needed.
     */
    @Override
    protected void configureWindow(java.awt.Window root) {
    }

    /**
     * A convenient static getter for the application instance.
     * @return the instance of FingyApp
     */
    public static FingyApp getApplication() {
        return Application.getInstance(FingyApp.class);
    }

    /**
     * Main method launching the application.
     */
    public static void main(String[] args) {
        launch(FingyApp.class, args);
    }
}
