/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package by.informatics.jcx.fingy.ide;

import java.awt.FontMetrics;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BoxView;
import javax.swing.text.ComponentView;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.IconView;
import javax.swing.text.LabelView;
import javax.swing.text.ParagraphView;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.TabSet;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;

public class FingyEditorKit extends StyledEditorKit {

    public static final int TABSTOP_SIZE = 4;
    private static final long serialVersionUID = 1L;
    private FingyDocument fingyDoc = new FingyDocument();
    private ViewFactory factory;
    private int tabWidthInPixels;

    public FingyEditorKit(FontMetrics fontMetrics) {
        tabWidthInPixels = fontMetrics.charWidth('w') * TABSTOP_SIZE;
    }

    @Override
    public Document createDefaultDocument() {
        return fingyDoc;
    }

    @Override
    public ViewFactory getViewFactory() {
        if (factory == null) {
            factory = new NoWrapViewFactory();
        }
        return factory;
    }

    public FingyDocument getFingyDocument() {
        return fingyDoc;
    }

    class NoWrapViewFactory implements ViewFactory {

        @Override
        public View create(Element elem) {
            String kind = elem.getName();
            if (kind != null) {
                if (kind.equals(AbstractDocument.ContentElementName)) {
                    return new LabelView(elem);
                } else if (kind.equals(AbstractDocument.ParagraphElementName)) {
                    return new NoWrapParagraphView(elem);
                } else if (kind.equals(AbstractDocument.SectionElementName)) {
                    return new BoxView(elem, View.Y_AXIS);
                } else if (kind.equals(StyleConstants.ComponentElementName)) {
                    return new ComponentView(elem);
                } else if (kind.equals(StyleConstants.IconElementName)) {
                    return new IconView(elem);
                }
            }

            // default to text display
            return new LabelView(elem);
        }
    }

    class NoWrapParagraphView extends ParagraphView {

        public NoWrapParagraphView(Element elem) {
            super(elem);
        }

        @Override
        public void layout(int width, int height) {
            super.layout(Short.MAX_VALUE, height);
        }

        @Override
        public float getMinimumSpan(int axis) {
            return super.getPreferredSpan(axis);
        }

        @Override
        public float nextTabStop(float x, int tabOffset) {
            TabSet tabs = getTabSet();
            if (tabs == null) {
                float d = (x - getTabBase()) / tabWidthInPixels;
                return getTabBase() + ((int) d + 1) * tabWidthInPixels;
            }
            return super.nextTabStop(x, tabOffset);
        }
    }
}
