/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ide;

import by.informatics.jcx.fingy.lang.Function;
import by.informatics.jcx.fingy.lang.ValueType;

public enum IOFunction {
    READ_BOOL ("read_bool", ValueType.BOOL),
    READ_INT ("read_int", ValueType.INT),
    READ_FLOAT ("read_float", ValueType.FLOAT),
    READ_STRING ("read_string", ValueType.STRING),
    PRINT ("print", ValueType.NULL),
    PRINTLN ("println", ValueType.NULL),
    ;

    public final String name;  // function identifier
    public final ValueType type;  // function return type

    private IOFunction(String id, ValueType type) {
        this.name = id;
        this.type = type;
    }

    public Function getFunction(FingyView owner) {
        switch (this) {
            case READ_BOOL:
            case READ_INT:
            case READ_FLOAT:
            case READ_STRING:
                return new ValueReader(owner.getFrame(), type, owner.getAction("stopAction"));
            case PRINT:
            case PRINTLN:
                return new ValueWriter(owner.getOutputPanel(), this);
        }

        return null;
    }
}
