/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ide;

import java.awt.FontMetrics;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.JEditorPane;
import javax.swing.text.Element;

public class JFingyPane extends JEditorPane {

    private static final long serialVersionUID = 1L;

    private FingyDocument fingyDoc;

    public JFingyPane() {
        super();
        
        FontMetrics fm = getFontMetrics(FingyDocument.FONT);
        setFont(FingyDocument.FONT);
        
        FingyEditorKit kit = new FingyEditorKit(fm);
        fingyDoc = kit.getFingyDocument();

        // register EditorKit for content type
        setEditorKitForContentType("text/fingy", kit);
        setContentType("text/fingy");

        // workaround for _something_ calling setVisible(false)
        // TODO: investigate what was calling setVisible(false)
        // check if this listener needed
        addFocusListener(new FocusAdapter() {

            @Override
            public void focusGained(FocusEvent e) {
                getCaret().setVisible(true);
            }
        });
    }
    public FingyDocument getFingyDocument() {
        return fingyDoc;
    }

    /*
     * Returns current line, indexing from 0.
     */
    public int getLine() {
        return fingyDoc.getDefaultRootElement().getElementIndex(
                getCaretPosition());
    }

    /*
     * Returns current column, indexing from 0.
     */
    public int getColumn() {
        int line = getLine();
        return getCaretPosition() - fingyDoc.getDefaultRootElement().
                getElement(line).getStartOffset();
    }

    /*
     * Returns position corresponding to start of the line.
     * Lines are indexed from 0.
     */
    public int getLinePosition(int line) {
        Element e = fingyDoc.getDefaultRootElement().getElement(line);
        if (e == null) {
            return -1;
        }
        return e.getStartOffset();
    }
    
    public int getLineByPosition(int position) {
        return fingyDoc.getDefaultRootElement().getElementIndex(position);
    }
    
    public int getLinesCount() {
        return fingyDoc.getDefaultRootElement().getElementCount();
    }
}
