/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ide;

import by.informatics.jcx.fingy.lang.Function;
import by.informatics.jcx.fingy.lang.NullValue;
import by.informatics.jcx.fingy.lang.VList;
import by.informatics.jcx.fingy.lang.VMContext;
import by.informatics.jcx.fingy.lang.VMError;
import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.ValueType;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;

class ValueReader implements Function {

    private final InputDialog dialog;
    private final Action stopAction;
    private Value value;

    public ValueReader(Frame owner, ValueType type, Action stopAction) {
        this.stopAction = stopAction;
        this.dialog = InputDialog.getDialog(owner, type);
    }

    public Value invoke(VMContext ctx, VList args) throws VMError {
        // TODO: better handling of exceptions thrown by invokeAndWait()
        final String message = (args.size() > 0) ? args.get(0).getString() : null;
        try {
            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {

                public void run() {
                    value = dialog.getValue(message);
                    if (value == NullValue.NULL) {
                        stopAction.actionPerformed(new ActionEvent(this,
                                ActionEvent.ACTION_PERFORMED, "Stop"));
                    } else {
                        System.out.println("[debug] ValueReader.invoke(): value = " + value.toString());
                    }
                }
            });
        } catch (InterruptedException ex) {
            Logger.getLogger(ValueReader.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvocationTargetException ex) {
            Logger.getLogger(ValueReader.class.getName()).log(Level.SEVERE, null, ex);
        }

        return value;
    }
}
