/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010. Alexey Rusakovich <mailto:exp_sm@yahoo.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.ide;

import by.informatics.jcx.fingy.lang.Function;
import by.informatics.jcx.fingy.lang.NullValue;
import by.informatics.jcx.fingy.lang.VList;
import by.informatics.jcx.fingy.lang.VMContext;
import by.informatics.jcx.fingy.lang.VMError;
import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.ValueType;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ValueWriter implements Function {

    private final OutputPanel outputPanel;
    private String values = null;
    private IOFunction function;

    public ValueWriter(OutputPanel outputPanel, IOFunction function) {
        this.outputPanel = outputPanel;
        this.function = function;
    }

    @Override
    public Value invoke(VMContext ctx, VList args) throws VMError {
        List<Value> list = args.asList();
        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < list.size(); i++) {
            Value val = list.get(i);
            if (val.getType() == ValueType.STRING) {
                sb.append(val.getString());
            } else {
                sb.append(val.getObject().toString());
            }
        }
        if (function == IOFunction.PRINTLN)
        {
            sb.append("\n");
        }
        values = sb.toString();

        // TODO: better handling of exceptions thrown by invokeAndWait()
        try {
            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {

                @Override
                public void run() {
                    outputPanel.append(values);
                }
            });
        } catch (InterruptedException ex) {
            Logger.getLogger(ValueWriter.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvocationTargetException ex) {
            Logger.getLogger(ValueWriter.class.getName()).log(Level.SEVERE, null, ex);
        }

        return NullValue.NULL;
    }
}
