/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import java.util.ArrayList;
import java.util.List;

public class ArrayValue extends Value {

    public static final int VISIBLE_ELEMENTS_COUNT = 10;
    public static final int MAX_VISIBLE_RECURSION = 3;

    private static final String ARRAY_DOTS = "...";
    private static final String ARRAY_DELIMETER = ", ";
    private static final String ARRAY_PREFIX = "{";
    private static final String ARRAY_SUFFIX = "}";

    private final List<Value> value;

    @Deprecated
    public ArrayValue(List<Value> value) {
        this.value = new ArrayList<Value>(value);
    }

    public static ArrayValue create(List<Value> value) {
        return new ArrayValue(value);
    }

    public ArrayValue() {
        this.value = new ArrayList<Value>();
    }

    public List<Value> getValue() {
        return value;
    }

    @Override
    public Object getObject() {
        return value;
    }

    @Override
    public List<Value> getList() {
        return value;
    }

    @Override
    public ValueType getType() {
        return ValueType.ARRAY;
    }

    private String toString(int deep) {
        if (deep <= 0) {
            return ARRAY_DOTS;
        }
        StringBuilder sb = new StringBuilder();
        sb.append(ARRAY_PREFIX);
        if (!value.isEmpty()) {
            sb.append(valueToString(value.get(0), deep));
            int cnt = Math.min(VISIBLE_ELEMENTS_COUNT, value.size());
            for (int i = 1; i < cnt; i++) {
                sb.append(ARRAY_DELIMETER);
                sb.append(valueToString(value.get(i), deep));
            }
            if (value.size() != cnt) {
                sb.append(ARRAY_DELIMETER + ARRAY_DOTS);
            }
        }
        sb.append(ARRAY_SUFFIX);
        return sb.toString();
    }

    private static final String valueToString(Value value, int deep) {
        if (value instanceof ArrayValue) {
            return ((ArrayValue) value).toString(deep - 1);
        } else {
            return value.toString();
        }
    }

    @Override
    public String toString() {
        return toString(MAX_VISIBLE_RECURSION);
    }
}
