/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

public class IntValue extends Value {

    private static int MIN_FVALS = -999;
    private static int MAX_FVALS = 9999;

    private static IntValue[] fvals = new IntValue[MAX_FVALS - MIN_FVALS + 1];
    static {
        for (int i = MIN_FVALS; i <= MAX_FVALS; ++i) {
            fvals[i - MIN_FVALS] = new IntValue(i);
        }
    }

    private long value;

    @Deprecated
    public IntValue(long value) {
        this.value = value;
    }

    public static IntValue create(long value) {
        int ivalue = (int) value;
        if (ivalue >= MIN_FVALS && ivalue <= MAX_FVALS) {
            return fvals[ivalue - MIN_FVALS];
        } else {
            return new IntValue(value);
        }
    }

    public long getValue() {
        return value;
    }

    @Override
    public long getLong() {
        return value;
    }

    @Override
    public Object getObject() {
        return value;
    }

    @Override
    public double getDouble() {
        return value;
    }

    public ValueType getType() {
        return ValueType.INT;
    }


    @Override
    public boolean equals(Object other) {
        if (this == other) {
            return true;
        }
        if (!(other instanceof IntValue)) {
            return false;
        }
        return this.value == ((IntValue) other).value;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 29 * hash + (int) (this.value ^ (this.value >>> 32));
        return hash;
    }
}
