/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

public class InvalidValueTypeError extends VMError {

    private static final long serialVersionUID = 0L;

    public InvalidValueTypeError() {
        super("Invalid type.");
    }

    public InvalidValueTypeError(ValueType type, ValueType... expected) {
        super(formatMessage(type, expected));
    }

    public InvalidValueTypeError(String message) {
        super(message);
    }

    private static String formatMessage(ValueType type, ValueType[] expected) {
        if (expected.length == 0) {
            return String.format("Invalid type \"%s.\"", type);
        } else if (expected.length == 1) {
            return String.format("Invalid type \"%s\", expected \"%s\".",
                    type.getTypeName(), expected[0].getTypeName());
        } else {
            StringBuilder sb = new StringBuilder();
            sb.append("Invalid type \"").append(type.getTypeName());
            sb.append("\". Expected one of \"");
            sb.append(expected[0].name().toLowerCase()).append("\"");
            for (int i = 1; i < expected.length; ++i) {
                sb.append(", \"");
                sb.append(expected[i].getTypeName()).append("\"");
            }
            sb.append(".");
            return sb.toString();
        }
    }
}
