/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import by.informatics.jcx.fingy.lang.parser.TokenMgrError;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LexicalError extends SyntaxError {

    private static final long serialVersionUID = 1L;

    public LexicalError(int line, String msg) {
        super(line, msg);
    }

    private static int extractLine(TokenMgrError error) {
        String msg = error.getMessage();
        Pattern p = Pattern.compile(".*line ([0-9]+).*");
        Matcher m = p.matcher(msg);
        if (m.find()) {
            return Integer.parseInt(m.group(1));
        } else {
            return -1;
        }
    }

    private static String extractMessage(TokenMgrError error) {
        String msg = error.getMessage();
        Pattern p = Pattern.compile("^.* (Encountered: .*)$");
        Matcher m = p.matcher(msg);
        if (m.find()) {
            return m.group(1);
        } else {
            return null;
        }
    }

    public static LexicalError fromTokenMgrError(TokenMgrError error) {
        int line = extractLine(error);
        String msg = extractMessage(error);
        return new LexicalError(line, msg);
    }
}
