/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import java.util.Collections;
import java.util.Map;

public class SimpleModule implements Module {

    private Map<String, Function> functions;
    private Map<String, Value> values;

    public SimpleModule(
            Map<String, Function> functions,
            Map<String, Value> values) {
        if (functions == null) {
            functions = Collections.emptyMap();
        }
        if (values == null) {
            values = Collections.emptyMap();
        }
        this.functions = functions;
        this.values = values;
    }

    @Override
    public Function getFunction(String name) {
        return functions.get(name);
    }

    @Override
    public Value getValue(String name) {
        return values.get(name);
    }

    @Override
    public String[] getFunctionNames() {
        return functions.keySet().toArray(new String[0]);
    }

    @Override
    public String[] getValueNames() {
        return values.keySet().toArray(new String[0]);
    }
}
