/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import java.util.LinkedHashMap;
import java.util.Map;

public class StringValue extends Value {

    public static final StringValue EMPTY_STRING = new StringValue("");

    private static final int MAX_POOL_SIZE = 1000;

    private static final Map<String, StringValue> pool =
            new LinkedHashMap<String, StringValue>(MAX_POOL_SIZE, 0.5f) {

        private static final long serialVersionUID = 1L;

        @Override
        protected boolean removeEldestEntry(Map.Entry<String, StringValue> eldest) {
            return size() > MAX_POOL_SIZE;
        }
    };

    private String value;
    private String printable;

    @Deprecated
    public StringValue(String value) {
        this.value = value;
        printable = toPrintable(value);
    }

    public static StringValue create(String value) {
        if (value.length() == 0) {
            return EMPTY_STRING;
        } else if (pool.containsKey(value)) {
            return pool.get(value);
        } else {
            StringValue v = new StringValue(value);
            pool.put(value, v);
            return v;
        }
    }

    public String getValue() {
        return value;
    }

    @Override
    public Object getObject() {
        return value;
    }

    @Override
    public ValueType getType() {
        return ValueType.STRING;
    }

    @Override
    public String getString() {
        return this.value;
    }

    @Override
    public String toString() {
        return printable;
    }

    private static String toPrintable(String str) {
        str = str.replaceAll("\n", "\\n");
        if (!str.contains("'")) {
            return String.format("\'%s\'", str);
        } else {
            return String.format("\"%s\"", str.replaceAll("\"", "\\\""));
        }
    }

    @Override
    public boolean equals(Object other) {
        if (this == other) {
            return true;
        }
        if (!(other instanceof StringValue)) {
            return false;
        }
        return this.value.equals(((StringValue) other).value);
    }

    @Override
    public int hashCode() {
        return value.hashCode();
    }
}
