/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class VList implements Iterable<Value> {

    private Value[] values;

    public VList(Value[] values) {
        this.values = values.clone();
    }

    public VList(List<Value> values) {
        this.values = (Value[]) values.toArray();
    }

    public Value get(int index) throws VMError {
        if (index < 0 || index >= values.length) {
            throw new InvalidParametersCountError();
        }
        return values[index];
    }

    public List<Value> asList() {
        return Collections.unmodifiableList(Arrays.asList(values));
    }

    public int size() {
        return values.length;
    }

    public void checkCount(int count) throws VMError {
        if (size() != count) {
            throw new InvalidParametersCountError();
        }
    }

    public void checkMinCount(int minCount) throws VMError {
        if (size() < minCount) {
            throw new InvalidParametersCountError();
        }
    }

    public void checkMaxCount(int maxCount) throws VMError {
        if (size() > maxCount) {
            throw new InvalidParametersCountError();
        }
    }

    public Iterator<Value> iterator() {
        return Arrays.asList(values).iterator();
    }
}
