/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang;

import java.util.List;

public abstract class Value {

    public double getDouble() throws VMError {
        throw new InvalidValueTypeError(getType());
    }

    public long getLong() throws VMError {
        throw new InvalidValueTypeError(getType());
    }

    public boolean getBoolean() throws VMError {
        throw new InvalidValueTypeError(getType());
    }

    public List<Value> getList() throws VMError {
        throw new InvalidValueTypeError(getType());
    }

    public String getString() throws VMError {
        throw new InvalidValueTypeError(getType());
    }

    public Object getObject() {
        throw new AbstractMethodError();
    }

    public int getInt() throws VMError {
        long val = getLong();
        if (val > Integer.MAX_VALUE || val < Integer.MIN_VALUE) {
            throw new InvalidValueTypeError("Too big integer.");
        }
        return (int) val;
    }

    public float getFloat() throws VMError {
        return (float) getDouble();
    }

    public abstract ValueType getType();

    @Override
    public String toString() {
        return getObject().toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Value)) {
            return false;
        }
        return ((Value) obj).getObject().equals(this.getObject());
    }

    @Override
    public int hashCode() {
        return getObject().hashCode();
    }
}
