/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.parser;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public abstract class ASTNode {

    public final int line;
    public final int column;

    ASTNode(Token token) {
        this.line = token.beginLine;
        this.column = token.beginColumn;
    }

    public ASTNode(int line) {
        this.line = line;
        this.column = -1;
    }

    public ASTNode(ASTNode node) {
        this.line = node.line;
        this.column = node.column;
    }
    
    public ASTNode() {
        this(-1);
    }

    public abstract void visit(Visitor paramVisitor);

    protected static final <Node extends ASTNode> List<Node> asList(Node[] nodes) {
        return Collections.unmodifiableList(Arrays.asList(nodes));
    }

    protected static final <Node extends ASTNode> List<Node> asList(List<Node> nodes) {
        return Collections.unmodifiableList(nodes);
    }
}
