/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.parser;

class ParserHelper {

    static final String unquoteString(String raw) {
        String quote = raw.substring(0, 1);
        raw = raw.substring(1, raw.length() - 1).replace("\\" + quote, quote);
        return raw;
    }

    static boolean containsToken(
            int[][] expectedTokenSequences, int kind) {
        for (int i = 0; i < expectedTokenSequences.length; ++i) {
            if (expectedTokenSequences[i][0] == kind) {
                return true;
            }
        }
        return false;
    }

    static String getTokenImage(String[] tokenImages, Token token) {
        int kind = token.kind;
        String image = " \"" + addEscapes(token.image) + '"';
        switch (kind) {
            case FingyParserConstants.STRING:
            case FingyParserConstants.IDENT:
            case FingyParserConstants.INTEGER:
            case FingyParserConstants.REAL:
                return getTokenImage(tokenImages, kind) + image;
            default:
                return getTokenImage(tokenImages, kind);
        }
    }

    static String getTokenImage(String[] tokenImages, int kind) {
        switch (kind) {
            case 0:
                return "end of file";
            case FingyParserConstants.EOL:
                return "end of line";
            case FingyParserConstants.STRING:
                return "string";
            case FingyParserConstants.IDENT:
                return "identifier";
            case FingyParserConstants.INTEGER:
                return "integer";
            case FingyParserConstants.REAL:
                return "float";
            default:
                return tokenImages[kind];
        }
    }

    static String addEscapes(String string) {
        return ParseException.add_escapes(string);
    }
}
