/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.ArrayValue;
import by.informatics.jcx.fingy.lang.VMError;
import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.IndexOutOfRangeError;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

class ArrayHelper implements InternalFunction {

    public static final int INIT_CALL_ID = 20;
    public static final int APPEND_CALL_ID = 21;
    public static final int GET_CALL_ID = 25;
    public static final int SET_CALL_ID = 26;
    private static ArrayHelper INSTANCE = new ArrayHelper();

    public void invoke(int code, VMExecutor e) throws VMError {
        Value v, g;
        List<Value> vl;
        switch (code) {
            case INIT_CALL_ID:
                e.push(new ArrayValue());
                break;
            case APPEND_CALL_ID:
                v = e.pop();
                e.peek().getList().add(v);
                break;
            case GET_CALL_ID:
                vl = getList(e);
                v = e.pop();
                try {
                    for (Value i : vl) {
                        v = v.getList().get((int) i.getLong());
                    }
                } catch (IndexOutOfBoundsException exception) {
                    throw new IndexOutOfRangeError();
                }
                e.push(v);
                break;
            case SET_CALL_ID:
                vl = getList(e);
                v = e.pop();
                g = e.pop();
                try {
                    for (int i = 0; i < vl.size() - 1; i++) {
                        v = v.getList().get((int) vl.get(i).getLong());
                    }
                    v.getList().set((int) vl.get(vl.size() - 1).getLong(), g);
                } catch (IndexOutOfBoundsException exception) {
                    throw new IndexOutOfRangeError();
                }
                break;
            default:
                throw new InternalVMError("Invalid ForLoopHelper code");
        }
    }

    private ArrayList<Value> getList(VMExecutor e) throws VMError {
        int s = e.getFArg();
        ArrayList<Value> res = new ArrayList<Value>(s);
        for (int i = 0; i < e.getFArg(); ++i) {
            res.add(e.pop());
        }
        Collections.reverse(res);
        return res;
    }

    public static void register(VMExecutor e) {
        e.addIFunc(INSTANCE, INIT_CALL_ID);
        e.addIFunc(INSTANCE, APPEND_CALL_ID);
        e.addIFunc(INSTANCE, GET_CALL_ID);
        e.addIFunc(INSTANCE, SET_CALL_ID);
    }
}
