/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.SyntaxError;
import by.informatics.jcx.fingy.lang.parser.ASTNode;
import by.informatics.jcx.fingy.lang.parser.FingyParser;
import by.informatics.jcx.fingy.lang.parser.ParseException;

class AsmCodeGen {

    public static void main(String args[]) {

        FingyParser parser;
        java.io.InputStream input;

        if (args.length == 1) {
            String fname = args[args.length - 1];
            try {
                input = new java.io.FileInputStream(fname);
            } catch (java.io.FileNotFoundException e) {
                System.out.println("File not found.");
                return;
            }
        } else if (args.length == 0) {
            input = System.in;
        } else {
            System.out.println("Usage: java FingyParser [input file]");
            return;
        }

        parser = new FingyParser(input);
        try {
            ASTNode root = parser.parse();
            System.out.println("Successful parse!");
            ConstPoolBuilder cp = new ConstPoolBuilder();
            CodeGenerator cg = new DebugCodeGenerator(System.out);
            NamespaceBuilder nm = new NamespaceBuilder();
            nm.addName("print");
            CodeBuilderVisitor visitor = new CodeBuilderVisitor(cg, cp, nm);
            cg.start();
            root.visit(visitor);
            VarPoolBuilder vpb = nm.createPoolBuilder();
            vpb.createVarPool();
            try {
                visitor.rethrowErrors();
            } catch (SyntaxError ex) {
                System.out.print(String.format(
                        "SyntaxError on line %1s: ", ex.getLineNumber()));
                System.out.println(ex.getMessage());
            }
            cg.finish();

        } catch (ParseException e) {
            String msg = e.getMessage();
            msg = msg.substring(0, msg.indexOf("\n"));
            System.out.println("Syntax Error: " + msg);
        }
    }
}
