/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.Value;

class ConstPool {

    private Value[] pool;
    private static final int cshift = Const.values().length + 1;

    ConstPool() {
    }

    public final Value get(int id) throws InternalVMError {
        try {
            if (id < cshift) {
                return Const.values()[id].constValue;
            } else {
                return pool[id - cshift];
            }
        } catch (ArrayIndexOutOfBoundsException ex) {
            throw new InternalVMError("ConstPool: invalid const id.");
        }
    }

    public void put(int id, Value value) throws InternalVMError {
        if (id < cshift) {
            throw new InternalVMError("ConstPool: rewriting builtin constant.");
        }
        try {
            pool[id - cshift] = value;
        } catch (ArrayIndexOutOfBoundsException ex) {
            throw new InternalVMError("ConstPool: storing const with invalid id.");
        }
    }

    void setSize(int count) {
        assert pool == null;
        pool = new Value[count - cshift + 1];
    }
}
