/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.Value;
import java.util.HashMap;
import java.util.Map;

class ConstPoolBuilder {

    private Map<Value, Integer> index = new HashMap<Value, Integer>();

    private int count = Const.values().length;
    private ConstPool product = new ConstPool();

    private class ConstArgCode implements ArgCodeProvider {

        private final int code;
        private final Value value;

        ConstArgCode(int code, Value value) {
            this.code = code;
            this.value = value;
        }

        public int getCode() {
            return code;
        }

        @Override
        public String toString() {
            return String.format("C%1d %2s", code, value.toString());
        }
    }

    public ArgCodeProvider add(Value v) {
        Integer cid = index.get(v);
        int id;
        if (cid == null) {
            id = newConst(v);
        } else {
            id = cid.intValue();
        }
        return new ConstArgCode(id, v);
    }

    private int newConst(Value v) {
        int cid = ++count;
        index.put(v, cid);
        return cid;
    }

    ConstPool createConstPool() throws InternalVMError {
        return product;
    }

    public void fillConstPool() throws InternalVMError {
        product.setSize(count);
        for (Value val: index.keySet()) {
            product.put(index.get(val), val);
        }
    }
}
