/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import java.io.PrintStream;

class DebugCodeGenerator extends AbstractCodeGenerator {

    private PrintStream out;
    private int labelCounter;

    public DebugCodeGenerator(PrintStream out) {
        this.out = out;
    }

    @Override
    public void addOp(Operation op) {
        out.print('\t');
        out.print(op);
        if (op.extraArgument) {
            out.print("\targ?");
        }
        out.println();
    }

    @Override
    public void addOp(Operation op, int arg) {
        printOp(op, Integer.toString(arg));
    }

    @Override
    public void setLabel(ArgCodeProvider label) {
        out.print(label);
        out.println(":");
    }

    @Override
    public void addOp(Operation op, Enum<?> arg) {
        printOp(op, arg.toString());
    }

    @Override
    public void start() {
        out.println("> START");
    }

    @Override
    public void finish() {
        out.println("> END");
    }

    public void addOp(Operation op, ArgCodeProvider arg) {
        printOp(op, arg != null ? arg.toString() : "<null>");
    }

    private void printOp(Operation op, String arg) {
        out.print('\t');
        out.print(op);
        if (op.extraArgument) {
            out.print('\t');
            out.print(arg);
        } else {
            out.print("\targ!");
        }
        out.println();
    }

    public ArgCodeProvider newLabel() {

        return new ArgCodeProvider() {

            private final int label = labelCounter++;

            public int getCode() {
                return label;
            }

            @Override
            public String toString() {
                return String.format("LB %1d", label);
            }
        };
    }
}
