/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

enum Operation {

    // var&fun operators
    CLOAD   (true),  // load const
    FARG    (true),  // make function args list
    FCALL,           // call function
    ICALL   (true),  // internal call
    VSTOR   (true),  // stor variable
    VLOAD   (true),  // load variable
    FDEF    (true),  // 'define' function
    IMPORT  (true),  // import module
    NOTIFY  (true),  // notify variable change

    // list operations
    GOTO    (true),  // goto label
    GOIF    (true),  // goto label if
    GONIF   (true),  // goto label unless

    // math operations
    NEG, ADD, SUB, MUL, DIV, POW, POS,

    // logical operations
    NOT, AND, OR, XOR,

    // relation operations
    EQ, LT, GT,

    // misc
    SWAP,   // swap two values on stack top
    DUP,    // duplicate value on stack top
    LINE    (true),
    EXIT,   // exit from script
    RET,    // return from function
    POP,
    NAME    (true),
    UVAR    (true),
    ;

    public final boolean extraArgument;

    Operation() {
        this(false);
    }

    Operation(boolean arg) {
        extraArgument = arg;
    }
}
