/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.Function;
import by.informatics.jcx.fingy.lang.NullValue;
import by.informatics.jcx.fingy.lang.RuntimeError;
import by.informatics.jcx.fingy.lang.SyntaxError;
import by.informatics.jcx.fingy.lang.VList;
import by.informatics.jcx.fingy.lang.VMContext;
import by.informatics.jcx.fingy.lang.VMError;
import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.parser.ASTNode;
import by.informatics.jcx.fingy.lang.parser.FingyParser;
import by.informatics.jcx.fingy.lang.parser.ParseException;

class TestInterpreter {

    public static void main(String[] args) {
        FingyParser parser;
        java.io.InputStream input;

        if (args.length == 1) {
            String fname = args[args.length - 1];
            try {
                input = new java.io.FileInputStream(fname);
            } catch (java.io.FileNotFoundException e) {
                System.out.println("File not found.");
                return;
            }
        } else if (args.length == 0) {
            input = System.in;
        } else {
            System.out.println("Usage: java FingyParser [input file]");
            return;
        }

        parser = new FingyParser(input);
        ASTNode root = null;
        try {
            root = parser.parse();
            System.out.println("Successful parse!");
        } catch (ParseException e) {
            String msg = e.getMessage();
            msg = msg.substring(0, msg.indexOf("\n"));
            System.out.println("Syntax Error: " + msg);
            return;
        }

        NamespaceBuilder nm = new NamespaceBuilder();
        ExternalFunction fPrint = new ExternalFunction(new Function() {

            public Value invoke(VMContext ctx, VList args) throws VMError {
                System.out.print("print: ");
                for (int i = 0; i < args.size(); ++i) {
                    System.out.print(" ");
                    System.out.print(args.get(i));
                }
                System.out.println();
                return NullValue.NULL;
            }
        });

        nm.addName("print", new InternalValue(fPrint, InternalValueType.EFUNCTION));
        ConstPoolBuilder cpb = new ConstPoolBuilder();
        VMCodeBuilder bd = new VMCodeBuilder(nm, cpb);
        cpb.fillConstPool();
        VMCode code = null;
        try {
            code = bd.generateCode(root);
        } catch (SyntaxError ex) {
            System.out.print(String.format(
                    "SyntaxError on line %1s: ", ex.getLineNumber()));
            System.out.println(ex.getMessage());
            return;
        }

        VMMachine exec = new VMMachine(null, code);
        try {
            exec.run();
        } catch (RuntimeError ex) {
            System.out.print("VM error: ");
            System.out.println(ex.getMessage());
            return;
        }
    }
}
