/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.Module;
import by.informatics.jcx.fingy.lang.SyntaxError;
import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.parser.ASTNode;
import java.util.List;
import java.util.Map;

public class VMCode {

    int[] code;
    VarPoolBuilder vpb;
    ConstPool cp;

    VMCode(List<Integer> code, ConstPool cp, VarPoolBuilder vpb) {
        this.code = new int[code.size()];
        for (int i = 0; i < code.size(); ++i) {
            this.code[i] = code.get(i);
        }
        this.cp = cp;
        this.vpb = vpb;
    }

    public final static VMCode create(
            ASTNode src,
            Map<String, Module> modules,
            boolean debug) throws SyntaxError {
        NamespaceBuilder nm = new NamespaceBuilder();

        for (String mname : new String[] {"$builtin", "$usermodule"}) {
            if (modules.containsKey(mname)) {
                Module defm = modules.get(mname);
                String[] names;
                names = defm.getValueNames();
                if (names != null) {
                    for (String name : names) {
                        if (!nm.containsLocalName(name)) {
                            nm.addName(name, defm.getValue(name));
                        }
                    }
                }
                names = defm.getFunctionNames();
                if (names != null) {
                    for (String name : defm.getFunctionNames()) {
                        ExternalFunction ef =
                                new ExternalFunction(defm.getFunction(name));
                        Value v = new InternalValue(ef,
                                InternalValueType.EFUNCTION);
                        if (!nm.containsName(name)) {
                            nm.addName(name, v);
                        }
                    }
                }
            }
        }

        if (modules != null) {
            for (String name : modules.keySet()) {
                Module m = modules.get(name);
                Value v = new InternalValue(m, InternalValueType.MODULE);
                nm.addName("module$" + name, v);
            }
        }

        ConstPoolBuilder cpb = new ConstPoolBuilder();
        VMCodeBuilder cb = new VMCodeBuilder(nm, cpb);

        VMCode res = cb.generateCode(src);
        cpb.fillConstPool();
        return res;
    }

    final static VMCode create(
            ASTNode src,
            NamespaceBuilder nm,
            ConstPoolBuilder cpb)
            throws SyntaxError {
        VMCodeBuilder cb = new VMCodeBuilder(nm, cpb);
        return cb.generateCode(src);
    }
}
