/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.SyntaxError;
import by.informatics.jcx.fingy.lang.parser.ASTNode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

class VMCodeBuilder {

    private NamespaceBuilder nm;
    private List<Integer> code = new ArrayList<Integer>();
    private ConstPoolBuilder cpb;

    class Label implements ArgCodeProvider {

        private int code;

        public int getCode() {
            return code;
        }

        void setCode(int code) {
            this.code = code;
        }
    }

    class CodeGenerator extends AbstractCodeGenerator {

        private Map<ArgCodeProvider, List<Integer>> providers =
                new HashMap<ArgCodeProvider, List<Integer>>();

        public void addOp(Operation op) {
            code.add(op.ordinal());
        }

        public void addOp(Operation op, int arg) {
            code.add(op.ordinal());
            code.add(arg);
        }

        public void addOp(Operation op, ArgCodeProvider arg) {
            code.add(op.ordinal());
            if (!providers.containsKey(arg)) {
                providers.put(arg, new ArrayList<Integer>());
            }
            providers.get(arg).add(code.size());
            code.add(-1);
        }

        public void start() {
            providers.clear();
        }

        public void finish() {
            for (ArgCodeProvider arg : providers.keySet()) {
                int c = arg.getCode();
                for (Integer coord : providers.get(arg)) {
                    code.set(coord, c);
                }
            }
        }

        public ArgCodeProvider newLabel() {
            return new Label();
        }

        public void setLabel(ArgCodeProvider label) {
            Label lb = (Label) label;
            lb.setCode(code.size());
        }
    }

    VMCodeBuilder(NamespaceBuilder nm, ConstPoolBuilder cpb) {
        this.nm = nm;
        this.cpb = cpb;
    }


    public VMCode generateCode(ASTNode source) throws SyntaxError {
        CodeGenerator cg = new CodeGenerator();
        CodeBuilderVisitor visitor = new CodeBuilderVisitor(cg, cpb, nm);
        cg.start();
        source.visit(visitor);
        visitor.rethrowErrors();
        cg.addOp(Operation.CLOAD, Const.NULL);
        cg.addOp(Operation.RET);
        cg.finish();
        return new VMCode(code, cpb.createConstPool(), nm.createPoolBuilder());
    }
}
