/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import java.util.Collection;
import java.util.List;

import by.informatics.jcx.fingy.lang.RuntimeError;
import by.informatics.jcx.fingy.lang.VMContext;
import by.informatics.jcx.fingy.lang.VMError;
import by.informatics.jcx.fingy.lang.Variable;

public final class VMMachine {

    public static final int STACK_LIMIT = 10000;
    
    VMExecutor executor;
    VMContext vmcontext;
    private VMCode code;
    private boolean isStarted;
    private boolean debug;

    public VMMachine(VMContext vmcontext, VMCode code) {
        this.code = code;
        this.vmcontext = vmcontext;
        this.executor = createExecutor();
    }

    public boolean run() throws RuntimeError {
        try {
            isStarted = true;
            return executor.run();
        } catch (VMError ex) {
            throw getError(ex);
        }
    }

    public void pause() {
        executor.pause();
    }

    public void setDebug(boolean debug) {
        boolean oldd = this.debug;
        this.debug = debug;
        if (oldd != debug) {
            executor = createExecutor();
        }
    }

    public boolean isRunning() {
        return isStarted && !executor.isStopped();
    }

    public boolean runLine() throws RuntimeError {
        try {
            isStarted = true;
            return executor.runLine();
        } catch (VMError ex) {
            throw getError(ex);
        }
    }

    public int getStackDeep() {
        return executor.getStackDeep();
    }

    public int getLine() {
        return executor.getLine();
    }

    public List<Collection<Variable>> getVariables() throws RuntimeError {
        try {
            return executor.getVariables();
        } catch (VMError ex) {
            throw getError(ex);
        }
    }

    private RuntimeError getError(VMError vmerr) {
        return new RuntimeError(executor.getLine(), vmerr.getMessage());
    }

    private VMExecutor createExecutor() {
        VMExecutor ex = new VMExecutor(this, code, debug);
        ForLoopHelper.register(ex);
        ArrayHelper.register(ex);
        return ex;
    }

    public boolean isPaused() {
        return executor.isPaused();
    }
}
