/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import java.util.ArrayList;
import java.util.List;

import by.informatics.jcx.fingy.lang.Value;
import by.informatics.jcx.fingy.lang.Variable;
import by.informatics.jcx.fingy.lang.VariableChangeListener;

public class VPVariable implements Variable {

    private final VarPool vpool;
    private final String name;
    private final int id;
    private boolean valid;
    private List<VariableChangeListener> listeners;

    VPVariable(VarPool vpool, String name, int id) {
        this.vpool = vpool;
        this.name = name;
        this.id = id;
        this.valid = true;
    }

    public Value getValue() {
        return vpool.get(id);
    }

    public void setValue(Value value) {
        vpool.put(id, value);
        fireChange();        
    }

    public String getName() {
        return name;
    }

    void setValid(boolean valid) {
        this.valid = valid;
    }

    public boolean isValid() {
        return valid;
    }

    @Override
    public String toString() {
        return String.format("Variable %1s = %2s", name, getValue());
    }

    @Override
    public void addListener(VariableChangeListener listener) {
        if (listeners == null) {
            listeners = new ArrayList<VariableChangeListener>(1);
        }
        listeners.add(listener);
    }

    @Override
    public void removeListener(VariableChangeListener listener) {
        if (listeners != null) {
            listeners.remove(listener);
            if (listeners.isEmpty()) {
                listeners = null;
            }
        }
    }

    void fireChange() {
        if (listeners != null) {
            for (int i = 0; i < listeners.size(); ++i) {
                listeners.get(i).variableChanged(this);
            }
        }
    }
}
