/*
 *  Fingy. Educational programming environment for children.
 *  Copyright (C) 2010.  Zhlobich Andrei <mailto:anjensan@informatics.by>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package by.informatics.jcx.fingy.lang.vm;

import by.informatics.jcx.fingy.lang.Value;
import java.util.List;

class VarPoolBuilder {

    private final int maxid;
    private final Integer[] exid;
    private final Value[] exval;
    private final int shift;

    VarPoolBuilder(int maxid, int shift, List<Integer> exid, List<Value> exval) {
        this.maxid = maxid;
        this.shift = shift;
        if (exid != null) {
            this.exid = exid.toArray(new Integer[0]);
            this.exval = exval.toArray(new Value[0]);
        } else {
            this.exid = null;
            this.exval = null;
        }
    }

    VarPoolBuilder(int maxid, int shift) {
        this(maxid, shift, null, null);
    }

    public VarPool createVarPool() {
        return fillVP(new VarPool(maxid));
    }

    public VarPool createVarPool(VarPool vp) {
        VarPool p;
        if (vp == null) {
            p = new VarPool(maxid);
        } else {
            p = new VarPool(maxid, vp, shift);
        }
        return fillVP(p);
    }

    private VarPool fillVP(VarPool vp) {
        if (exid != null) {
            for (int i = 0; i < exid.length; ++i) {
                vp.putAbsolute(exid[i], exval[i]);
            }
        }
        return vp;
    }
}
